<?php

define('RBSPAYMENT_PAYMENT_NAME', 'Berekebank');

define('RBSPAYMENT_PROD_URL' , 'https://securepayments.berekebank.kz/payment/rest/');
define('RBSPAYMENT_TEST_URL', 'https://3dsec.berekebank.kz/payment/rest/');

define('RBSPAYMENT_ENABLE_LOGGING', true);
define('RBSPAYMENT_ENABLE_CALLBACK', true);

define('RBSPAYMENT_CURRENCY_CODES', serialize(array(
    'USD' => '840',
    'UAH' => '980',
    'RUR' => '810',
    'RUB' => '643',
    'RON' => '946',
    'KZT' => '398',
    'KGS' => '417',
    'JPY' => '392',
    'GBR' => '826',
    'EUR' => '978',
    'CNY' => '156',
    'BYR' => '974',
    'BYN' => '933'
)));

//this close tag must be here -> ?><?php
class BerekeBank
{
public $prod_url;
public $test_url;
public $alternative_domain = null;
public $berekebank_enable_cart_options = false;
public $berekebank_enable_refund_options = false;
public $logging = RBSPAYMENT_ENABLE_LOGGING;
public $language = 'en';
public $version = '3.1.8';
public $token;
public $login;
public $password;
public $mode;
public $stage;
public $send_cart;
public $versionFfd;
public $paymentMethodType;
public $paymentObjectType;
public $paymentMethodTypeDelivery;
public $taxSystem;
public $taxType;
public $discountHelper;
public $enable_back_url_settings = false;
public $api_version = 1;
public $allowCallbacks;
public $berekebank_cacert_enable = true;
public $berekebank_cacert_path = null;
public function __construct($config = null)
{
if ($config) {
$this->token = isset($config['berekebank_merchantToken']) ? $config['berekebank_merchantToken'] : null;
$this->login = isset($config['berekebank_merchantLogin']) ? $config['berekebank_merchantLogin'] : null;
$this->password = isset($config['berekebank_merchantPassword']) ? $config['berekebank_merchantPassword'] : null;
$this->mode = isset($config['berekebank_mode']) ? $config['berekebank_mode'] : null;
$this->stage = isset($config['berekebank_stage']) ? $config['berekebank_stage'] : null;
$this->send_cart = isset($config['berekebank_send_cart']) ? $config['berekebank_send_cart'] : false;
$this->versionFfd = isset($config['berekebank_version_ffd']) ? $config['berekebank_version_ffd'] : null;
$this->paymentMethodType = isset($config['berekebank_paymentMethodType']) ? $config['berekebank_paymentMethodType'] : null;
$this->paymentMethodTypeDelivery = isset($config['berekebank_paymentMethodTypeDelivery']) ? $config['berekebank_paymentMethodTypeDelivery'] : null;
$this->paymentObjectType = isset($config['berekebank_paymentObjectType']) ? $config['berekebank_paymentObjectType'] : null;
$this->taxSystem = isset($config['berekebank_taxSystem']) ? $config['berekebank_taxSystem'] : null;
$this->taxType = isset($config['berekebank_taxType']) ? $config['berekebank_taxType'] : null;
}
$this->test_url = RBSPAYMENT_TEST_URL;
$this->prod_url = RBSPAYMENT_PROD_URL;
if (defined('RBSPAYMENT_API_VERSION')) {
$this->api_version = RBSPAYMENT_API_VERSION;
}
if (defined('RBSPAYMENT_PROD_URL_ALTERNATIVE_DOMAIN')) {
$this->alternative_domain = RBSPAYMENT_PROD_URL_ALTERNATIVE_DOMAIN;
}
$this->allowCallbacks = $this->_allowCallbacks();
if (defined('RBSPAYMENT_ENABLE_CART_OPTIONS')) {
$this->berekebank_enable_cart_options = RBSPAYMENT_ENABLE_CART_OPTIONS;
if ($this->berekebank_enable_cart_options === true){
$this->library('BerekeBankDiscount');
$this->discountHelper = new BerekeBankDiscount();
}
} else {
$this->berekebank_enable_cart_options = false;
}
if (file_exists(DIR_SYSTEM . "library/berekebank_cacert.cer")) {
$this->berekebank_cacert_enable = true;
$this->berekebank_cacert_path = DIR_SYSTEM . "library/berekebank_cacert.cer";
} else {
$this->berekebank_cacert_enable = false;
}
if (defined('RBSPAYMENT_ENABLE_REFUND_TAB') && RBSPAYMENT_ENABLE_REFUND_TAB === true) {
$this->berekebank_enable_refund_options = true;
} else {
$this->berekebank_enable_refund_options = false;
}
if (defined('RBSPAYMENT_ENABLE_BACK_URL_SETTINGS') && RBSPAYMENT_ENABLE_BACK_URL_SETTINGS === true) {
$this->enable_back_url_settings = true;
}
}
/**
* @return string
*/
public function getTestUrl()
{
return $this->test_url;
}
/**
* @return string
*/
public function getProdUrl()
{
if (defined('RBSPAYMENT_PROD_URL_ALTERNATIVE_DOMAIN') && defined('RBSPAYMENT_PROD_URL_ALT_PREFIX')) {
if (substr($this->login, 0, strlen(RBSPAYMENT_PROD_URL_ALT_PREFIX)) == RBSPAYMENT_PROD_URL_ALT_PREFIX) {
$pattern = '/^https:\/\/[^\/]+/';
return preg_replace($pattern, rtrim(RBSPAYMENT_PROD_URL_ALTERNATIVE_DOMAIN, '/'), $this->prod_url);
} else {
}
} elseif (defined('RBSPAYMENT_PROD_URL_ALTERNATIVE_DOMAIN')) {
$pattern = '/^https:\/\/[^\/]+/';
return preg_replace($pattern, rtrim(RBSPAYMENT_PROD_URL_ALTERNATIVE_DOMAIN, '/'), $this->prod_url);
}
return $this->prod_url;
}
/**
* @return string
*/
public function getAlternativeDomain()
{
return $this->alternative_domain;
}
/**
* @return mixed
*/
public function getVersionFfd()
{
return $this->versionFfd;
}
/**
* @param $delivery
* @return mixed
*/
public function getPaymentMethodType($delivery = false)
{
if ($delivery) {
return $this->paymentMethodTypeDelivery;
}
return $this->paymentMethodType;
}
/**
* @return mixed
*/
public function getPaymentObjectType()
{
return $this->paymentObjectType;
}
/**
* @return string
*/
public function getDefaultMeasurement()
{
if ($this->versionFfd == "v1_05") {
return RBSPAYMENT_MEASUREMENT_NAME;
}
return RBSPAYMENT_MEASUREMENT_CODE;
}
/**
* @param $property
* @param $value
* @return $this
*/
public function __set($property, $value)
{
if (property_exists($this, $property)) {
$this->$property = $value;
}
return $this;
}
/**
* @param $data
* @param $action_address
* @param array $headers
* @return string
*/
public function _sendGatewayData($data, $action_address, $headers = array())
{
$curl_opt = array(
CURLOPT_HTTPHEADER => $headers,
CURLOPT_VERBOSE => true,
CURLOPT_SSL_VERIFYHOST => false,
CURLOPT_URL => $action_address,
CURLOPT_RETURNTRANSFER => true,
CURLOPT_POST => true,
CURLOPT_POSTFIELDS => $data,
CURLOPT_HEADER => true,
);
$ssl_verify_peer = false;
if ($this->berekebank_cacert_path != null && $this->berekebank_cacert_enable == true) {
$ssl_verify_peer = true;
$curl_opt[CURLOPT_CAINFO] = $this->berekebank_cacert_path;
}
$curl_opt[CURLOPT_SSL_VERIFYPEER] = $ssl_verify_peer;
$ch = curl_init();
curl_setopt_array($ch, $curl_opt);
$response = curl_exec($ch);
if ($response === false) {
$error = array('errorCode' => 999, "errorMessage" => curl_error($ch));
return json_encode($error);
}
$header_size = curl_getinfo($ch, CURLINFO_HEADER_SIZE);
curl_close($ch);
return substr($response, $header_size);
}
/**
*
* @param string $order_number
* @param integer $amount
* @param integer $currency_code
* @param string $return_url
* @param null $orderBundle
* @return mixed[]
*/
public function register_order($order_number, $amount, $return_url, $currency_code = null, $orderBundle = null)
{
$data = array(
'orderNumber' => $order_number . "_" . time(),
'amount' => $amount,
'returnUrl' => $return_url,
'jsonParams' => json_encode(
[
'CMS:' => 'Opencart ' . VERSION,
'Module-Version: ' => $this->version
]
),
);
if (!empty($currency_code)) {
$data['currency'] = $currency_code;
}
if ($this->send_cart && !empty($orderBundle)) {
$data['taxSystem'] = $this->taxSystem;
$data['orderBundle']['orderCreationDate'] = date('c');
$data['orderBundle'] = json_encode($orderBundle);
}
$response = $this->gateway($this->stage == 'two' ? 'registerPreAuth.do' : 'register.do', $data);
return $response;
}
/**
*
* @param string $method
* @param mixed[] $data
* @return mixed[]
*/
public function gateway($method, $data) {
if (!empty($this->token)) {
$decoded_credentials = base64_decode($this->token);
list($l, $p) = explode(':', $decoded_credentials);
$data['userName'] = $l;
$data['password'] = $p;
} else {
$data['userName'] = $this->login;
$data['password'] = $this->password;
}
if ($this->mode == 'test') {
$action_address = $this->test_url . $method;
} else {
$action_address = $this->getProdUrl() . $method;
}
$headers = array('CMS: OpenCart ' . VERSION, 'Module-Version: ' . $this->version);
$response = $this->_sendGatewayData($data, $action_address, $headers);
$this->logger($action_address, $data, $response);
return $response;
}
/**
*
* @param string $url
* @param string $method
* @param mixed[] $request
* @param mixed[] $response
* @return integer
*/
public function logger($url, $request, $response) {
$this->library('log');
$file_name = "oc23x_berekebank_" . date("y-m") . ".log";
$logger = new Log($file_name);
$logger->write("REQUEST (" .$url ."):\n" . json_encode($request) . "\nRESPONSE: " . $response . "\n\n");
}
/**
*
* @param string $orderId
* @return mixed[]
*/
public function get_order_status($orderId) {
if ($this->mode == 'test') {
$action_address = $this->test_url . "getOrderStatusExtended.do";
} else {
$action_address = $this->getProdUrl() . "getOrderStatusExtended.do";
}
if (!empty($this->token)) {
$decoded_credentials = base64_decode($this->token);
list($l, $p) = explode(':', $decoded_credentials);
$data['userName'] = $l;
$data['password'] = $p;
} else {
$data['userName'] = $this->login;
$data['password'] = $this->password;
}
$data['orderId'] = $orderId;
$response = $this->_sendGatewayData($data, $action_address);
$this->logger($action_address, $data, $response);
return $response;
}
public function _allowCallbacks() {
$this->library('log');
$file_name = "oc23x_berekebank_" . date("y-m") . ".log";
$logger = new Log($file_name);
if (defined('RBSPAYMENT_PROD_URL_ALTERNATIVE_DOMAIN')
&& defined('RBSPAYMENT_PROD_URL_ALT_PREFIX')
&& (substr($this->login, 0, strlen(RBSPAYMENT_PROD_URL_ALT_PREFIX)) != RBSPAYMENT_PROD_URL_ALT_PREFIX)
) {
return false;
}
return RBSPAYMENT_ENABLE_CALLBACK;
}
private function library($library) {
$file = DIR_SYSTEM . 'library/' . str_replace('../', '', (string)$library) . '.php';
if (file_exists($file)) {
include_once($file);
} else {
trigger_error('Error: Could not load library ' . $file . '!');
exit();
}
}
function get_numeric_currency_code($currency_code) {
$currency_codes = array(
'BYN' => '933',
'BHD' => '048',
'BYR' => '974',
'CAD' => '124',
'CNY' => '156',
'EUR' => '978',
'GBP' => '826',
'HKD' => '344',
'HUF' => '348',
'ILS' => '376',
'JPY' => '392',
'KGS' => '417',
'KRW' => '410',
'KZT' => '398',
'MDL' => '498',
'MYR' => '458',
'OMR' => '512',
'PHP' => '608',
'RON' => '946',
'RUB' => '643',
'RUR' => '810',
'SGD' => '702',
'UAH' => '980',
'USD' => '840',
'NGN' => '566',
'MZN' => '943',
'BGN' => '975',
'BZD' => '084',
'GHS' => '936',
'GNF' => '324',
'XOF' => '952',
'PLN' => '985',
'LSL' => '426',
'TZS' => '834',
'NZD' => '554',
'KHR' => '116',
'TRY' => '949',
'AMD' => '051',
'SAR' => '682',
'AED' => '784',
'COP' => '170',
'AUD' => '036',
'IDR' => '360',
'KWD' => '414',
'JOD' => '400',
'INR' => '356'
);
return isset($currency_codes[$currency_code]) ? $currency_codes[$currency_code] : null;
}
}